/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/


#include "context_graphicitem.h"
#include "MediaItem/covercache.h"

#include "mediaitem.h"
#include "engineplayer.h"
//#include "debug.h"

#include <QString>
#include <QObject>

/*
********************************************************************************
*                                                                              *
*    Class Header                                                          *
*                                                                              *
********************************************************************************
*/
Header::Header()
{
    setAcceptsHoverEvents(true);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);

    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
    
    _width  = 100;
    _height = 26;
}

void Header::setText(const QString& t)
{
    m_text = t;
}

QRectF Header::boundingRect() const
{
    int width = _width < 100 ? 100 : _width;

    return QRectF(0, 0, width, _height);
}

// Inherited from QGraphicsLayoutItem
void Header::setGeometry(const QRectF &geom)
{
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF Header::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return boundingRect().size();
}



void Header::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
Q_UNUSED(widget)

    //QColor m_brush_color = QApplication::palette().color(QPalette::Button);

    QColor m_brush_color = QColor(0xfca822).lighter(140);
//     QColor m_brush_color = QColor( 255, 98, 25);
    m_brush_color.setAlpha(153);
    
    QRect rect = boundingRect().toRect();
    painter->setPen(QPen( m_brush_color, 0.1, Qt::SolidLine, Qt::RoundCap));
    painter->setBrush(QBrush( m_brush_color ,Qt::SolidPattern));
    painter->drawRoundedRect(rect, 4.0, 4.0);

    //! draw text
    QFont font = QApplication::font();
    font.setBold( true );
    painter->setFont(font);
//     painter->setPen(QApplication::palette().color(QPalette::Normal,QPalette::WindowText)) ;
    painter->setPen(QColor(Qt::white));

    QFontMetrics metric( painter->font() );
    const QString elided = metric.elidedText ( m_text, Qt::ElideRight, _width -50);
    
    painter->drawText(rect, Qt::AlignCenter | Qt::AlignVCenter, elided);
}


/*
********************************************************************************
*                                                                              *
*    Class WebLinkItem                                                         *
*       -> use with    webLinkItem->setHtml("<a href='text'>http://url</a>");  *
*                                                                              *
********************************************************************************
*/
WebLinkItem::WebLinkItem()
{
    setAcceptsHoverEvents(false);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIsFocusable, false);
    setCursor(Qt::PointingHandCursor);

    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
}

void WebLinkItem::clear()
{
    m_link.clear();
    m_text.clear();
    m_pixmap = QPixmap();
}

void WebLinkItem::setPixmap(const QPixmap& pixmap )
{
    m_pixmap = pixmap.scaled(QSize(18,18), Qt::KeepAspectRatio, Qt::SmoothTransformation);
}

void WebLinkItem::setText(const QString& text)
{
    m_text = text;
}
void WebLinkItem::setLink(const QString& link)
{
    m_link = link;
}

QRectF WebLinkItem::boundingRect() const
{
    QFont font;
    font.setPointSize( 8 );
    font.setUnderline(true);

    QFontMetrics fm( font );
    return QRectF(0, 0, fm.width(m_text) + m_pixmap.width()  + 12, 26);
}

// Inherited from QGraphicsLayoutItem
void WebLinkItem::setGeometry(const QRectF &geom)
{
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF WebLinkItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return boundingRect().size();
}

void WebLinkItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
Q_UNUSED(widget)
    QRect rect = boundingRect().toRect();

    //! draw pixmap
    painter->drawPixmap(4,4,m_pixmap);

    //! draw text
    QFont font;
    font.setPointSize( 8 );
    font.setUnderline(true);

    painter->setFont(font);
    //painter->setPen(QColor(0x1767b7));
    painter->setPen(QApplication::palette().color(QPalette::Normal, QPalette::Link));

    painter->drawText(rect.adjusted(m_pixmap.width()+2, 0, 0, 0), Qt::AlignCenter | Qt::AlignVCenter, m_text);
}

void WebLinkItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    QDesktopServices::openUrl(QUrl(m_link));
}


/*
********************************************************************************
*                                                                              *
*    Class TitleGraphicItem                                                    *
*                                                                              *
********************************************************************************
*/
TitleGraphicItem::TitleGraphicItem()
{
    setAcceptsHoverEvents(false);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
    m_width = 630;

    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
}

QRectF TitleGraphicItem::boundingRect() const
{
    return QRectF(0, 0, m_width, 40);
}

void TitleGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
    //! draw categorie text
    QFont font = QApplication::font();
    font.setBold(true);
    font.setPointSize(14);

    painter->setFont( font );

    QColor inlineColor  = QColor(0xfca822);
    QColor outlineColor = QColor (inlineColor.lighter(140) );

    int Xpos = 0;
    painter->setPen(inlineColor);
    painter->drawStaticText(Xpos,5, QStaticText(m_name));

    //BEGIN: horizontal line
    {
      QPoint start(boundingRect().toRect().left()+Xpos, boundingRect().toRect().top()+28);
      start.rx() += 3;
      QPoint horizontalGradTop(boundingRect().toRect().topLeft());
      horizontalGradTop.rx() += boundingRect().toRect().width() - 6;
      QLinearGradient gradient(start, horizontalGradTop);
      gradient.setColorAt(0, inlineColor);
      gradient.setColorAt(1, outlineColor);
      painter->fillRect(QRect(start, QSize(boundingRect().toRect().width() - 6, 1)), gradient);
    }
    //END: horizontal line
}

// Inherited from QGraphicsLayoutItem
void TitleGraphicItem::setGeometry(const QRectF &geom)
{
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF TitleGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return boundingRect().size();
}


/*
********************************************************************************
*                                                                              *
*    Class TagGraphicItem                                                      *
*                                                                              *
********************************************************************************
*/
TagGraphicItem::TagGraphicItem()
{
    setAcceptsHoverEvents(false);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);

    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
}

QRectF TagGraphicItem::boundingRect() const
{
    QFont font;
    font.setPointSize( 10 );

    QFontMetrics metric( font );

    QSize size = metric.boundingRect(m_name).size();

    return QRectF(0,0, 6 + size.width() + 6, 2 + size.height() + 2);
}

void TagGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
    QRect rect = boundingRect().toRect();
    painter->setPen(QPen( QColor( 231, 231, 231,150), 0.1, Qt::SolidLine, Qt::RoundCap));
    painter->setBrush(QBrush( QColor( 231, 231, 231) ,Qt::SolidPattern));
    painter->drawRoundedRect(rect, 4.0, 4.0);

    //! draw text
    QFont font;
    font.setPointSize( 10 );
    painter->setFont( font );
    painter->setPen(QColor(Qt::black));
    painter->drawText(rect.adjusted(2, 0, -4, 0), Qt::AlignCenter | Qt::AlignVCenter, m_name);
}

void TagGraphicItem::updateItem()
{
    prepareGeometryChange();

    updateGeometry();

    // update painting
    update();
}

// Inherited from QGraphicsLayoutItem
void TagGraphicItem::setGeometry(const QRectF &geom)
{
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF TagGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which);
Q_UNUSED(constraint);
    return boundingRect().size();
}


/*
********************************************************************************
*                                                                              *
*    Class TextGraphicItem                                                     *
*                                                                              *
********************************************************************************
*/
TextGraphicItem::TextGraphicItem()
{
    setAcceptsHoverEvents(false);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    //setFlag(QGraphicsItem::ItemIgnoresTransformations, true);

    m_width  = 630;
    m_height = 200;
    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );

    setFont( QFont("Arial", 9, QFont::Normal) );
    //setDefaultTextColor(QColor(0x555555));
    setDefaultTextColor(QApplication::palette().color(QPalette::Disabled, QPalette::WindowText));
    setTextInteractionFlags(Qt::TextBrowserInteraction);
    setOpenExternalLinks(false);


    setFlag(QGraphicsItem::ItemIsFocusable, false);
}

void TextGraphicItem::clear()
{
    this->document()->clear();
    updateItem();
}

void TextGraphicItem::updateItem()
{
    prepareGeometryChange();

    setTextWidth(m_width - 40);
    m_height = this->document()->size().height();

    //Debug::debug() << "### TextGraphicItem height " << m_height;

    updateGeometry();

    // update painting
    update();
}

QRectF TextGraphicItem::boundingRect() const
{
    return QRectF(0, 0, m_width, m_height);
}


// Inherited from QGraphicsLayoutItem
void TextGraphicItem::setGeometry(const QRectF &geom)
{
    QGraphicsLayoutItem::setGeometry(geom);
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF TextGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which)
Q_UNUSED(constraint)
    return QSizeF( boundingRect().size());
}

void TextGraphicItem::contextMenuEvent ( QGraphicsSceneContextMenuEvent * event )
{
  event->ignore();
}


/*
********************************************************************************
*                                                                              *
*    Class AlbumThumbGraphicItem                                               *
*                                                                              *
********************************************************************************
*/
AlbumThumbGraphicItem::AlbumThumbGraphicItem()
{
    setAcceptsHoverEvents(false);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);

    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );

    m_pix = QPixmap(":/images/emptycoverblue110x110.png");
    m_pix = m_pix.scaled(QSize(64,64), Qt::KeepAspectRatio, Qt::FastTransformation);
}


void AlbumThumbGraphicItem::clear()
{
    m_title.clear();
    m_year.clear();

    m_pix = QPixmap(":/images/emptycoverblue110x110.png");
    m_pix = m_pix.scaled(QSize(64,64), Qt::KeepAspectRatio, Qt::FastTransformation);
}

void AlbumThumbGraphicItem::updateItem()
{
    update();
}

QRectF AlbumThumbGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 90, 90);
}

// Inherited from QGraphicsLayoutItem
void AlbumThumbGraphicItem::setGeometry(const QRectF &geom)
{
    QGraphicsLayoutItem::setGeometry(geom);
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF AlbumThumbGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which)
Q_UNUSED(constraint)
    return QSizeF( boundingRect().size());
}

void AlbumThumbGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
    QPixmap pixTemp(QSize(90,90));
    {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);

      p.drawPixmap(13,9, m_pix);
      p.end();
    }

   painter->drawPixmap(0, 0, pixTemp);

   //! paint album title
   painter->setFont(QFont("Arial", 7, QFont::Normal));
   painter->setPen(QApplication::palette().color(QPalette::Disabled, QPalette::WindowText));

   QFontMetrics m1( painter->font());
   const QString elided_album = m1.elidedText ( m_title, Qt::ElideRight, 88);
   painter->drawText(QRect (0,72,88,20), Qt::AlignVCenter | Qt::AlignHCenter, elided_album );

   //! paint year
   //painter->setFont(QFont("Arial", 7, QFont::Normal));
   painter->setPen(QApplication::palette().color(QPalette::Disabled, QPalette::WindowText));
   painter->drawText(QRect(0, 0, 90, 10), Qt::AlignTop | Qt::AlignCenter, m_year);
}


/*
********************************************************************************
*                                                                              *
*    Class NowPlayingGraphicItem                                               *
*                                                                              *
********************************************************************************
*/
NowPlayingGraphicItem::NowPlayingGraphicItem()
{
    setAcceptsHoverEvents(false);
    setAcceptDrops(false);
    setFlag(QGraphicsItem::ItemIsSelectable, false);
    setFlag(QGraphicsItem::ItemIsMovable, false);
    setFlag(QGraphicsItem::ItemIgnoresTransformations, true);

    setGraphicsItem(this);

    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
}

void NowPlayingGraphicItem::updateItem()
{
    update();
}

QRectF NowPlayingGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 500, 180);
}

// Inherited from QGraphicsLayoutItem
void NowPlayingGraphicItem::setGeometry(const QRectF &geom)
{
    QGraphicsLayoutItem::setGeometry(geom);
    setPos(geom.topLeft());
}

// Inherited from QGraphicsLayoutItem
QSizeF NowPlayingGraphicItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
Q_UNUSED(which)
Q_UNUSED(constraint)
    return QSizeF(boundingRect().size());
}

void NowPlayingGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)

  const int width = 500;

  //! Get data
  MEDIA::TrackPtr track = EnginePlayer::instance()->playingTrack();
  QPixmap         pix   = CoverCache::instance()->cover(track);

  //! font
  QFont font_normal = QApplication::font();
  font_normal.setBold(false);
  font_normal.setPointSize(font_normal.pointSize()*1.1);

  QFont font_bold   = QApplication::font();
  font_bold.setBold(true);
  font_bold.setPointSize(font_bold.pointSize()*1.1);

  //! -- Playing State
  if(EnginePlayer::instance()->engineState() != ENGINE::STOPPED && !EnginePlayer::instance()->playingUrl().isEmpty())
  {
    if(track->type() == TYPE_TRACK)
    {
        //! paint title
        painter->setPen(QApplication::palette().color(QPalette::Normal, QPalette::WindowText));
        painter->setFont(font_bold);
        QFontMetrics fm(font_bold);
        const QString titleTruncated = fm.elidedText ( track->title, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 15, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint album
        painter->setPen(QApplication::palette().color(QPalette::Disabled, QPalette::WindowText));
        painter->setFont(font_normal);
        QFontMetrics fm2(font_normal);
        const QString albumTruncated = fm2.elidedText ( track->album, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 35, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint artist
        const QString artistTruncated = fm2.elidedText ( track->artist, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 55, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );

        //! paint duration
        if(track->duration > 0) {
          const QString durationText = track->durationToString();
          const QString dureeTruncated = fm2.elidedText ( durationText, Qt::ElideRight, width-135 );
          painter->drawText(QRect(130, 75, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,dureeTruncated );
        }

/*        //! paint track rating
        ui_rating_widget->set_rating(media->data.rating);
        ui_rating_widget->show();
*/
        //! paint pixmap (coverart)
        painter->drawPixmap(10,10,110,110, pix);
    }
    else if(track->type() == TYPE_STREAM)
    {
        //! paint stream name
        painter->setPen(QApplication::palette().color(QPalette::Normal, QPalette::WindowText));
        painter->setFont(font_bold);
        QFontMetrics fm(font_bold);
        const QString streamTruncated = fm.elidedText ( track->name, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 15, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,streamTruncated );

        //! paint title
        painter->setPen(QApplication::palette().color(QPalette::Disabled, QPalette::WindowText));
        painter->setFont(font_normal);
        QFontMetrics fm2(font_normal);
        const QString title_or_url = track->title.isEmpty() ? track->url : track->title;

        const QString titleTruncated = fm2.elidedText ( title_or_url, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 35, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,titleTruncated );

        //! paint album
        const QString albumTruncated = fm2.elidedText ( track->album, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 55, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,albumTruncated );

        //! paint artist
        const QString artistTruncated = fm2.elidedText ( track->artist, Qt::ElideRight, width-135 );
        painter->drawText(QRect(130, 75, width-135, 20), Qt::AlignVCenter | Qt::AlignLeft,artistTruncated );

        //! paint pixmap (cover art)
        painter->drawPixmap(8,8,110,110, pix);
    }
  }
  else {
   painter->setPen(QApplication::palette().color(QPalette::Disabled, QPalette::WindowText));

   const QRect rect = boundingRect().toRect();

   painter->setFont(font_normal);

   painter->drawPixmap(rect.adjusted(10,-4,0,0).topLeft() ,QPixmap(":/images/info-message.png") );
   painter->drawText(QRect(70,6,250,22), Qt::AlignVCenter, QObject::tr("Player is stopped"));
  }
}
